
const mod = require('./../handlers/createApplication');
const graphqlMod = require('./../handler');
const jestPlugin = require('@lagenhetsbyte/serverless-jest-plugin');
const lodash = require('lodash');
const applicationService = require('./../services/applicationService');
const applicantService = require('./../services/applicantService');
const gql = require('graphql-tag');
const { print } = require('graphql/language/printer');

const { lambdaWrapper } = jestPlugin;
const wrapped = lambdaWrapper.wrap(mod, { handler: 'createApplication' });
const gqlWrapped = lambdaWrapper.wrap(graphqlMod, { handler: 'graphql' });

jest.mock('../util/authHeaderParser', () => {
  const parser = (e) => {
    const mockAdminContext = { user: { sub: '8cc60946-144d-493b-b148-3f1d44737730', email: 'admin.test@test.com', 'cognito:groups': ['admins'] } };
    const mockUserContext = { user: { sub: '1a17bc49-0a07-4ac8-aa36-39b9e3a01092', email: 'test@email.com' } };
    return e.admin ? mockAdminContext : mockUserContext;
  };
  return parser;
});

let mockApplicationId = 'To bee assigned';
const mockData = {
  'Password': 'X2FDzCteXDzSFXXM8LDS28TLTL7Mk3Lzw78WCSVaJeUDTuZYcWkz4xRN4XeCHFaGw83uBmhUbJncjEDpmHVrgmffE5c2ayLQ3AkNcTyPUbTmqqunvLYhktBbFemWcbCt',
  'Applicants': [
    {
      'ProfilePicture': 'https://www.ienglishstatus.com/wp-content/uploads/2018/04/Anonymous-Whatsapp-profile-picture.jpg',
      'PropositionPicture': 'https://circaoldhouses.com/wp-content/uploads/2017/10/key-hole-house-1.jpg',
      'Id': 2039301,
      'Members': {
        'Member1': {
          'Email': 'test@test.se',
          'Name': 'Kalle',
          'Ssn': '880404-0031',
          'Phone': '123123123',
          'ApartmentId': '203930.apartment1',
        },
        'Member2': null,
      },
      'Apartments': {
        'Apartment1': {
          'Landlord': null,
          'Address': 'Gratisgatan 12',
          'PostalCode': '16433',
          'City': 'Kista',
        },
        'Apartment2': null,
      },
    },
    {
      'ProfilePicture': 'https://www.ienglishstatus.com/wp-content/uploads/2018/04/Anonymous-Whatsapp-profile-picture.jpg',
      'PropositionPicture': 'https://circaoldhouses.com/wp-content/uploads/2017/10/key-hole-house-1.jpg',
      'Id': 902838,
      'Members': {
        'Member1': {
          'Email': '324110@lbtest.se',
          'Name': 'Josefine Holmberg',
          'Ssn': null,
          'Phone': '0739377805',
          'ApartmentId': '902838.apartment1',
        },
        'Member2': null,
      },
      'Apartments': {
        'Apartment1': {
          'Landlord': 'tingstr?ms',
          'Address': 'Rusth?llarv?gen 99',
          'PostalCode': '12843',
          'City': 'Bagarmossen',
        },
        'Apartment2': null,
      },
    },
  ],
};

describe('createApplication', () => {
  beforeAll((done) => {
    //  lambdaWrapper.init(liveFunction); // Run the deployed lambda
    done();
  });

  afterAll((done) => {
    const promises = [];
    promises.push(applicationService.delete(mockApplicationId));
    mockData.Applicants.forEach(a => promises.push(applicantService.delete(String(a.Id))));
    Promise.all(promises).then(() => {
      done();
    });
  });


  it('createApplication', done => wrapped.run({
    body: JSON.stringify(mockData),
  }).then((response) => {
    console.log(response.body);
    expect(response.statusCode).toBe(200);
    expect(response.body).toBeDefined();
    const body = JSON.parse(response.body);
    mockApplicationId = body.applicationId;

    done();
  })
    .catch(err => console.log(err)));

  it('createEpmtyApplication', done => wrapped.run({
    body: JSON.stringify({
      'Password': 'X2FDzCteXDzSFXXM8LDS28TLTL7Mk3Lzw78WCSVaJeUDTuZYcWkz4xRN4XeCHFaGw83uBmhUbJncjEDpmHVrgmffE5c2ayLQ3AkNcTyPUbTmqqunvLYhktBbFemWcbCt',
      'Applicants': [],
    }),
  }).then((response) => {
    expect(response.statusCode).toBe(400);
    done();
  }));

  const getGql = id => gql`{
      adminApplication(id: "${id}") {
        id
        applicants {
          id
          index
          image
        }
      }
    }`;
  it('adminApplication', done => gqlWrapped.run({
    body: {
      query: `${print(getGql(mockApplicationId))}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(response).toBeDefined();
    expect(response.statusCode).toBe(200);
    expect(body.errors).toBeFalsy();
    const applicants = lodash.get(body, 'data.adminApplication.applicants');
    expect(applicants).toBeDefined();
    applicants.forEach((applicant) => {
      expect(applicant.id).toEqual(String(mockData.Applicants[applicant.index].Id));
      expect(applicant.image).toEqual(String(mockData.Applicants[applicant.index].PropositionPicture));
    });
    done();
  })
    .catch(err => console.log(err)));
});
