const mod = require('./../handler');

const jestPlugin = require('@lagenhetsbyte/serverless-jest-plugin');

const { lambdaWrapper } = jestPlugin;
const wrapped = lambdaWrapper.wrap(mod, { handler: 'graphql' });
const lodash = require('lodash');
const adminService = require('../services/adminService');
const applicantService = require('../services/applicantService');
const documentService = require('../services/documentService');
const applicationService = require('../services/applicationService');
const uuid = require('uuid/v4');
const gql = require('graphql-tag');
const { print } = require('graphql/language/printer');
const { pushToDatabase } = require('../util/databaseGenerator');

const Chance = require('chance');

const chance = new Chance();

const createIds = (count) => {
  const idList = [];
  for (let index = 0; index < count; index++) {
    idList.push(uuid());
  }
  return idList;
};

const createEmails = (count) => {
  const idList = [];
  for (let index = 0; index < count; index++) {
    idList.push(chance.email());
  }
  return idList;
};

const mockAdmin = {
  cognitoId: '8cc60946-144d-493b-b148-3f1d44737730',
  email: 'admin.test@test.com',
  name: 'Adam Admin',
  phone: '+46123456789',
  picture: 'http://via.placeholder.com/150x150',
};

const applicationCount = 2;
const applicantsPerApplication = 3;

const applicationIds = createIds(applicationCount);
const applicantIds = createIds(applicationCount * applicantsPerApplication);
const documentIds = createIds(applicationCount * applicantsPerApplication);
const cognitoIds = createIds(applicationCount * applicantsPerApplication);
const emails = createEmails(applicationCount * applicantsPerApplication);
cognitoIds[0] = '1a17bc49-0a07-4ac8-aa36-39b9e3a01092';
emails[0] = 'test@email.com';

jest.mock('../util/authHeaderParser', () => {
  const parser = (e) => {
    const mockAdminContext = { user: { sub: '8cc60946-144d-493b-b148-3f1d44737730', email: 'admin.test@test.com', 'cognito:groups': ['admins'] } };
    const mockUserContext = { user: { sub: '1a17bc49-0a07-4ac8-aa36-39b9e3a01092', email: 'test@email.com' } };
    return e.admin ? mockAdminContext : mockUserContext;
  };
  return parser;
});

const applicationsQuery = require('./queries/applications');

describe('postGraphql', () => {
  beforeAll((done) => {
    pushToDatabase(applicantsPerApplication, applicationIds, applicantIds, emails, cognitoIds, documentIds, mockAdmin)
      .then(done());
  });

  afterAll((done) => {
    const promises = [];
    applicationIds.forEach(id => promises.push(applicationService.delete(id)));
    applicantIds.forEach(id => promises.push(applicantService.delete(id)));
    documentIds.forEach(id => promises.push(documentService.delete(id)));
    promises.push(adminService.delete(mockAdmin.cognitoId));

    Promise.all(promises).then(() => {
      done();
    });
  });

  const availabilityQuery = gql`{
    __schema {
      queryType {
        name
      }
    }
  }`;
  it('test endpoint availability', () => wrapped.run({
    body: {
      query: `${print(availabilityQuery)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    expect(response).toBeDefined();
    expect(response.statusCode).toBe(200);
  }));

  const adminApplicationsQuery = gql`{
    adminApplications(limit: 1) {
      items {
        id
      }
      nextToken
    }
  }`;
  it('adminApplications', () => wrapped.run({
    body: {
      query: `${print(adminApplicationsQuery)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
  }));

  const adminApplicationQuery = gql`{
    adminApplication(id: "${applicationIds[0]}") {
      id
      applicants {
        id
        email
      }
    }
  }`;
  it('adminApplication', () => wrapped.run({
    body: {
      query: `${print(adminApplicationQuery)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
  }));

  // // it('applicant', () => wrapped.run({ body: '{"query": "{__schema{queryType{name}}}"}' }).then((response) => {
  // //   expect(response).toBeDefined();
  // // }));

  const applicationQuery = gql`{
    application(id: "${applicantIds[0]}") {
      id
      applicants {
        id
        image
        email
        details {
          swapDate
          details1 {
            member {
              name
              ssn
              phone
              email
              registeredAddress
              registeredPostalCode
              registeredCity
            }
            apartment {
              
              landlord
              address
              postalCode
              city
            }
            
            personCount
            childrenCount
          }
          details2 {
            member {
              
              name
              ssn
              phone
              email
              registeredAddress
              registeredPostalCode
              registeredCity
            }
            apartment {
              
              landlord
              address
              postalCode
              city
            }
            
            personCount
            childrenCount
          }
        }
        swapReason {
          reason
          freeText
          approvalStatus {
            date
            type
            message
          }
        }
        documents {
          id
          fileName
          folder
        }
      }
    }
  }`;
  it('application', () => wrapped.run({
    body: {
      query: `${print(applicationQuery)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    // const testApplicant = body.data.application.applicants.find(a => a.email === emails[0]);
    expect(response.statusCode).toBe(200);
    expect(body.errors).toBeFalsy();
    // expect(testApplicant).toBeTruthy();
  }));

  it('applications', () => wrapped.run({
    body: {
      query: `${print(applicationsQuery)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
  }));

  const approveApartment = gql`
    mutation {
    approveApartment(applicantId: "${applicantIds[0]}", index: TWO) {
      approvalStatus {
        date
        type
        message
      }
    }
  }`;
  it('approveApartment', done => wrapped.run({
    body: {
      query: `${print(approveApartment)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'apartment2.approvalStatus.type')).toEqual('APPROVED');
    done();
  }));

  const approveMember = gql`
    mutation {
    approveMember(applicantId: "${applicantIds[0]}", index: TWO) {
      approvalStatus {
        date
        type
        message
      }
    }
  }`;
  it('approveMember', () => wrapped.run({
    body: {
      query: `${print(approveMember)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'member2.approvalStatus.type')).toEqual('APPROVED');
  }));

  const approveDetails = gql`
    mutation {
    approveDetails(applicantId: "${applicantIds[0]}", index: TWO) {
      approvalStatus {
        date
        type
        message
      }
    }
  }`;
  it('approveDetails', () => wrapped.run({
    body: {
      query: `${print(approveDetails)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'details2.approvalStatus.type')).toEqual('APPROVED');
  }));

  const approveDocumentMutation = gql`
    mutation {
      approveDocument(documentId: "${documentIds[2]}") {
        applicant {
          id
        }
      }
    }
  `;
  it('approveDocument', () => wrapped.run({
    body: {
      query: `${print(approveDocumentMutation)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return documentService.get(documentIds[2]);
  }).then((response) => {
    expect(response.approvalStatus.type).toBe('APPROVED');
  }));

  const approveSwapReasonMutation = gql`
    mutation {
      approveSwapReason(applicantId: "${applicantIds[0]}") {
        approvalStatus {
          type
        }
      }
    }
  `;
  it('approveSwapReason', () => wrapped.run({
    body: {
      query: `${print(approveSwapReasonMutation)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    expect(body.data.approveSwapReason.approvalStatus.type).toBe('APPROVED');
  }));

  // it('putApplicant', () => wrapped.run({ body: '{"query": "{__schema{queryType{name}}}"}' }).then((response) => {
  //   expect(response).toBeDefined();
  // }));

  // it('putApplication', () => wrapped.run({ body: '{"query": "{__schema{queryType{name}}}"}' }).then((response) => {
  //   expect(response).toBeDefined();
  // }));

  const putApartment = gql`
  mutation {
    putApartment(apartment: {
      applicantId: "${applicantIds[0]}",
      index: TWO,
      landlord: "CocaCola",
      address: "testväg 12",
      postalCode: "11322",
      city: "Stockholm"
    }) {
      landlord
      address
      postalCode
      city
    }
  }`;
  it('putApartment', done => wrapped.run({
    body: {
      query: `${print(putApartment)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'apartment2.postalCode')).toEqual('11322');
    done();
  }));

  const putDetails = gql`
  mutation {
    putDetails(
      applicantId: "${applicantIds[0]}",
      swapDate: "1254233"
      details1: {
        personCount: 2,
        moveToApplicant: "3454564",
        moveToApartment: ONE,
      }
      details2: {
        personCount: 20,
        moveToApplicant: "123",
        moveToApartment: TWO,
      }
    ) {
      swapDate
    }
  }`;
  it('putDetails', () => wrapped.run({
    body: {
      query: `${print(putDetails)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'details2.personCount')).toEqual(20);
  }));

  const putMember = gql`
  mutation {
    putMember(member: {
      applicantId: "${applicantIds[0]}"
      index: ONE
      name: "test"
      ssn: "12345890"
      phone: "phone"
      email: "test@gmail.com"
      registeredAddress: "Paradisäppelvägen 1337"
    }) {
      name
      ssn
      phone
      email
      registeredAddress
      registeredPostalCode
      registeredCity
    }
  }`;
  it('putMember', () => wrapped.run({
    body: {
      query: `${print(putMember)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'member1.email')).toEqual('test@gmail.com');
  }));

  const putSwapReasonMutation = gql`
  mutation {
    putSwapReason(swapReason: {
      applicantId: "${applicantIds[0]}"
      reason: ["Vill ha större lägenhet"]
      freeText: "whoop whoop!"
    }) {
      reason
    }
  }
  `;
  it('putSwapReason', () => wrapped.run({
    body: {
      query: `${print(putSwapReasonMutation)}`,
    },
    httpMethod: 'POST',
  })
    .then((response) => {
      const body = JSON.parse(response.body);
      expect(body.errors).toBeFalsy();
      expect(response.statusCode).toBe(200);
      expect(response).toBeDefined();
      return applicantService.get(applicantIds[0]);
    })
    .then((response) => {
      expect(response.swapReason).toBeDefined();
      expect(response.swapReason.reason).toEqual(['Vill ha större lägenhet']);
      expect(response.swapReason.approvalStatus.type).toBe('UPDATED');
    }));

  const setLastActive = gql`
    mutation {
      setLastActive(applicantId: "${applicantIds[0]}")
    }
    `;
  it('setLastActive', () => wrapped.run({
    body: {
      query: `${print(setLastActive)}`,
    },
    httpMethod: 'POST',
  })
    .then((response) => {
      const body = JSON.parse(response.body);
      expect(body.errors).toBeFalsy();
      expect(response.statusCode).toBe(200);
      expect(response).toBeDefined();
      return applicantService.get(applicantIds[0]);
    })
    .then((response) => {
      console.log(response.lastActive);
      expect(response.lastActive).toBeDefined();
    }));

  const setMemberSignLink = gql`
    mutation {
      setMemberSignLink(applicantId: "${applicantIds[0]}", index: ONE, link: "http://google.com") {
        name
        signLink
    }
  }`;
  it('setMemberSignLink', () => wrapped.run({
    body: {
      query: `${print(setMemberSignLink)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'member1.signLink')).toEqual('http://google.com');
  }));

  // const rejectApartment = gql`
  //   mutation {
  //   rejectApartment(applicantId: "${applicantIds[0]}", index: TWO, message: "inte bra") {
  //     approvalStatus {
  //       date
  //       type
  //       message
  //     }
  //   }
  // }`;
  // it('rejectApartment', done => wrapped.run({
  //   body: {
  //     query: `${print(rejectApartment)}`,
  //   },
  //   admin: true,
  //   httpMethod: 'POST',
  // }).then((response) => {
  //   const body = JSON.parse(response.body);
  //   expect(body.errors).toBeFalsy();
  //   expect(response.statusCode).toBe(200);
  //   expect(response).toBeDefined();
  //   return applicantService.get(applicantIds[0]);
  // }).then((response) => {
  //   expect(lodash.get(response, 'apartment2.approvalStatus.type')).toEqual('REJECTED');
  //   done();
  // }));

  const rejectMember = gql`
    mutation {
    rejectMember(applicantId: "${applicantIds[0]}", index: TWO, message: "inte bra") {
      approvalStatus {
        date
        type
        message
      }
    }
  }`;
  it('rejectMember', () => wrapped.run({
    body: {
      query: `${print(rejectMember)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'member2.approvalStatus.type')).toEqual('REJECTED');
  }));

  const rejectDetails = gql`
    mutation {
    rejectDetails(applicantId: "${applicantIds[0]}", index: TWO, message: "inte bra") {
      approvalStatus {
        date
        type
        message
      }
    }
  }`;
  it('rejectDetails', () => wrapped.run({
    body: {
      query: `${print(rejectDetails)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(lodash.get(response, 'details2.approvalStatus.type')).toEqual('REJECTED');
  }));

  const rejectDocumentMutation = gql`
    mutation {
      rejectDocument(documentId: "${documentIds[1]}") {
        applicant {
          id
        }
      }
    }
  `;
  it('rejectDocument', () => wrapped.run({
    body: {
      query: `${print(rejectDocumentMutation)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return documentService.get(documentIds[1]);
  }).then((response) => {
    expect(response.approvalStatus.type).toBe('REJECTED');
  }));

  const rejectSwapReasonMutation = gql`
    mutation {
      rejectSwapReason(applicantId: "${applicantIds[1]}") {
        approvalStatus {
          type
        }
      }
    }
  `;
  it('rejectSwapReason', () => wrapped.run({
    body: {
      query: `${print(rejectSwapReasonMutation)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    expect(body.data.rejectSwapReason.approvalStatus.type).toEqual('REJECTED');
  }));

  const removeDocumentMutation = gql`
    mutation {
      removeDocument(documentId: "${documentIds[0]}")
    }
  `;
  it('removeDocument', () => wrapped.run({
    body: {
      query: `${print(removeDocumentMutation)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return documentService.get(documentIds[0]);
  }).then((response) => {
    expect(response).toBeFalsy();
  }));

  const reviewApplicant = gql`
    mutation {
      reviewApplicant(applicantId: "${applicantIds[0]}") {
        id
      }
    }
  `;
  it('reviewApplicant', () => wrapped.run({
    body: {
      query: `${print(reviewApplicant)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(response.reviewed).toBeTruthy();
  }));

  const setApplicationStatus = gql`
    mutation {
      setApplicationStatus(applicationId: "${applicationIds[0]}", status: DECLINED) {
        id
      }
    }
  `;
  it('setApplicationStatus', () => wrapped.run({
    body: {
      query: `${print(setApplicationStatus)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicationService.get(applicationIds[0]);
  }).then((response) => {
    expect(response.status).toEqual('DECLINED');
  }));

  const inactivateApplication = gql`
    mutation {
      inactivateApplication(applicantId: "${applicantIds[0]}")
    }
  `;
  it('inactivateApplication', () => wrapped.run({
    body: {
      query: `${print(inactivateApplication)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    console.log(body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicationService.get(applicationIds[0]);
  }).then((response) => {
    expect(response.inactivatedBy).toEqual(applicantIds[0]);
  }));

  const reactivateApplication = gql`
    mutation {
      reactivateApplication(applicantId: "${applicantIds[0]}")
    }
  `;
  it('reactivateApplication', () => wrapped.run({
    body: {
      query: `${print(reactivateApplication)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    console.log(body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicationService.get(applicationIds[0]);
  }).then((response) => {
    expect(response.inactivatedBy).toEqual(null);
  }));

  const signApplicant = gql`
    mutation {
      signApplicant(applicantId: "${applicantIds[0]}") {
        id
      }
    }
  `;
  it('signApplicant', () => wrapped.run({
    body: {
      query: `${print(signApplicant)}`,
    },
    admin: true,
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    expect(body.errors).toBeFalsy();
    expect(response.statusCode).toBe(200);
    expect(response).toBeDefined();
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(response.reviewed).toBeTruthy();
  }));

  const uploadDocumentMutation = gql`
  mutation {
    uploadDocument(document: {
      applicantId: "${applicantIds[0]}"
      fileName: "newDocument.pdf"
      folder: "eu-west-1:random"
      type: ID_PROOF
    }) {
      id
      fileName
      folder
      type
      approvalStatus {
        type
      }
    }
  }
  `;
  it('uploadDocument', () => wrapped.run({
    body: {
      query: `${print(uploadDocumentMutation)}`,
    },
    httpMethod: 'POST',
  })
    .then((response) => {
      const body = JSON.parse(response.body);
      expect(body.errors).toBeFalsy();
      expect(response.statusCode).toBe(200);
      expect(response).toBeDefined();
      documentIds.push(lodash.get(body, 'data.uploadDocument.id'));
      return applicantService.get(applicantIds[0])
        .then((applicant) => {
          expect(applicant).toBeDefined();
          expect(applicant.documents.includes(lodash.get(body, 'data.uploadDocument.id')));
        });
    }));

  const deleteMember2 = gql`
  mutation {
    deleteMember2(applicantId: "${applicantIds[0]}")
  }`;
  it('deleteMember2', () => wrapped.run({
    body: {
      query: `${print(deleteMember2)}`,
    },
    httpMethod: 'POST',
  }).then((response) => {
    const body = JSON.parse(response.body);
    console.log(body);
    return applicantService.get(applicantIds[0]);
  }).then((response) => {
    expect(response.member2).toEqual(null);
  }));
});
