const applicationService = require('../services/applicationService');
const applicantService = require('../services/applicantService');
const moment = require('moment');
const { sendActivationMail } = require('../util/mails');

const lodash = require('lodash');


function getApplicationId(applicants) {
  const as = applicants.slice();
  let id = '';
  as.sort((a, b) => {
    if (a.Id < b.Id) return -1;
    if (a.Id > b.Id) return 1;
    return 0;
  });
  as.forEach((applicant) => {
    id += applicant.Id;
  });
  return id;
}

async function applicationExists(id) {
  const response = await applicationService.get(id);
  if (response) return true;
  return false;
}

function checkIf2to1(applicants) {
  let is2to1 = false;

  applicants.forEach(applicant => {
    const apartment2 = lodash.get(applicant, 'Apartments.Apartment2');
    if (apartment2) {
      is2to1 = true;
    }
  });

  return is2to1;
};

module.exports.createApplication = async (event, context, callback) => {
  try {
    const body = JSON.parse(event.body);

    if (lodash.get(body, 'Password') !== process.env.LEGACY_PASSWORD) throw Error('Not authorized');

    const applicants = lodash.get(body, 'Applicants');
    console.log('Initiating applicants');

    if (!applicants) throw Error('Invalid input');

    if (applicants.length === 0) throw Error('Received no applicants');
    console.log(JSON.stringify(applicants, null, 2));

    const applicationId = getApplicationId(applicants);
    const exists = await applicationExists(applicationId);

    if (exists) throw Error('Application already initiated');

    const application = await applicationService.put({
      id: applicationId,
      status: 'ACTIVE',
    });

    const is2to1Swap = checkIf2to1(applicants);
    console.log('Is 2 to 1 swap', is2to1Swap);

    const promises = [];
    const newApplicants = [];
    for (let i = 0; i < applicants.length; i++) {
      const applicant = applicants[i];
      const newApplicant = {
        id: String(applicant.Id),
        lastActive: moment.now(),
        index: i,
        application: application.id,
        swapReason: {},
        image: applicant.PropositionPicture ? applicant.PropositionPicture : null,
        profilePicture: applicant.ProfilePicture ? applicant.ProfilePicture : null,
        email: lodash.get(applicant, 'Members.Member1.Email'),
      };

      const members = [
        lodash.get(applicant, 'Members.Member1'),
        lodash.get(applicant, 'Members.Member2'),
      ];
      for (let j = 0; j < members.length; j++) {
        const member = members[j];
        const memberProp = `member${j + 1}`;
        const detailsProp = `details${j + 1}`;

        if (member || is2to1Swap) {
          newApplicant[memberProp] = {};
          newApplicant[memberProp].email = member && member.Email || null;
          newApplicant[memberProp].name = member && member.Name || null;
          newApplicant[memberProp].ssn = member && member.Ssn || null;
          newApplicant[memberProp].phone = member && member.Phone || null;
          if (member && member.ApartmentId) {
            const parts = member.ApartmentId.split('.');
            const index = parts[1] === 'apartment1' ? 'ONE' : 'TWO';
            newApplicant[detailsProp] = {
              applicantId: String(applicant.Id),
              memberIndex: index,
            };
          }
        }
      }

      const apartments = [
        lodash.get(applicant, 'Apartments.Apartment1'),
        lodash.get(applicant, 'Apartments.Apartment2'),
      ];
      for (let j = 0; j < apartments.length; j++) {
        const apartment = apartments[j];
        const apartmentProp = `apartment${j + 1}`;
        if (apartment) {
          newApplicant[apartmentProp] = {};
          if (apartment.Landlord) newApplicant[apartmentProp].landlord = apartment.Landlord ? apartment.Landlord : null;
          if (apartment.Address) newApplicant[apartmentProp].address = apartment.Address ? apartment.Address : null;
          if (apartment.PostalCode) newApplicant[apartmentProp].postalCode = apartment.PostalCode ? apartment.PostalCode : null;
          if (apartment.City) newApplicant[apartmentProp].city = apartment.city ? apartment.city : null;
        }
      }
      console.log(`Adding applicant ${JSON.stringify(newApplicant, null, 2)}`);
      promises.push(await applicantService.put(newApplicant));
      newApplicants.push(newApplicant);
    }
    console.log(JSON.stringify(promises, null, 2));
    await Promise.all(promises)
      .then(() => {
        console.log('Sending mail');
        sendActivationMail(newApplicants);
      });
    console.log('Sending success response');
    callback(null, {
      statusCode: 200,
      body: JSON.stringify({
        status: 'SUCCESS',
        applicationId: application.id,
      }),
    });
  } catch (error) {
    console.log(error);
    callback(null, {
      statusCode: 400,
      body: JSON.stringify({
        status: 'FAILED',
        message: error.message,
      }),
    });
  }
};
