const lodash = require('lodash');
const moment = require('moment');
const { combineResolvers } = require('graphql-resolvers');
const { baSendSignLinkEmail } = require('@lagenhetsbyte/message-service');
const applicantService = require('../services/applicantService');
const documentService = require('../services/documentService');
const applicationService = require('../services/applicationService');
const { 
  checkAndSendRejectionMail, 
  checkAndSendAllSigned,
  checkAndSendNotifyAdmin
} = require('../util/mails');


const { isAdmin, isAuthenticated, selfOrAdmin } = require('../util/authResolvers');


const assignProperty = async (property, data, applicantId, context) => {
  const applicant = await applicantService.get(applicantId);
  if (!applicant) throw Error('Applicant not found');
  if (lodash.get(context, 'user.email') !== applicant.email) throw Error('Not authorized');
  applicant[property] = data;
  await applicantService.putApplicant(applicant);
  return data;
};

const assignPropertyUpdateStatus = async (property, data, context) => {
  const applicant = await applicantService.get(data.applicantId);
  if (!applicant) throw Error('Applicant not found');
  if (lodash.get(context, 'user.email') !== applicant.email) throw Error('Not authorized');
  data.cognitoId = lodash.get(context, 'user.sub');
  delete data.applicantId;
  const approvalStatus = lodash.get(applicant[property], 'approvalStatus') ? lodash.get(applicant[property], 'approvalStatus') : {};
  approvalStatus.type = 'UPDATED';
  applicant.rejectNotified = false; // allow rejection emails again
  data.approvalStatus = approvalStatus;
  applicant[property] = data;
  await applicantService.putApplicant(applicant);
  return data;
};

const mergeProperty = async (property, data, context) => {
  const applicant = await applicantService.get(data.applicantId);
  if (!applicant) throw Error('Applicant not found');
  if (lodash.get(context, 'user.sub') !== applicant.cognitoId) throw Error('Not authorized');
  data.cognitoId = lodash.get(context, 'user.sub');
  delete data.applicantId;
  if (applicant[property] === null || applicant[property] === undefined) {
    lodash.assign(applicant[property], {});
  }
  lodash.merge(applicant[property], data);
  await applicantService.putApplicant(applicant);
  return data;
};

const mergePropertyUpdateStatus = async (property, data, context) => {
  const applicant = await applicantService.get(data.applicantId);
  if (!applicant) throw Error('Applicant not found');
  if (lodash.get(context, 'user.sub') !== applicant.cognitoId) throw Error('Not authorized');
  data.cognitoId = lodash.get(context, 'user.sub');
  delete data.applicantId;
  if (applicant[property] === null || applicant[property] === undefined) {
    lodash.assign(applicant[property], {});
  }
  const approvalStatus = lodash.get(applicant[property], 'approvalStatus') ? lodash.get(applicant[property], 'approvalStatus') : {};
  approvalStatus.type = 'UPDATED';
  applicant.rejectNotified = false; // allow rejection emails again
  data.approvalStatus = approvalStatus;
  applicant[property] = data;
  lodash.merge(applicant[property], data);
  await applicantService.putApplicant(applicant);
  await checkAndSendNotifyAdmin(applicant); // check statuses and notify admin if application seems done
  return data;
};

module.exports.applicants = combineResolvers(isAuthenticated, async (_, args, context) =>
  applicantService.getByEmail(lodash.get(context, 'user.email')));

module.exports.adminAnonymizeApplicant = combineResolvers(
  isAdmin,
  async (_, { email }) => {
    const applicants = await applicantService.anonymizeApplicants(email);
    return applicants;
  },
);

module.exports.putApplicant = combineResolvers(
  isAuthenticated,
  (_, { applicant }, context) => {
    applicant.email = lodash.get(context, 'user.email');
    applicant.started = false;
    applicant.rejectNotified = false; // allow rejection emails again
    return applicantService.putApplicant(applicant);
  }
);

module.exports.startApplicant = combineResolvers(isAuthenticated, async (_, { id }, context) => {
  const applicant = await applicantService.get(id);
  if (!applicant) throw Error('Applicant not found');
  if (applicant.email !== lodash.get(context, 'user.email')) throw Error('Not authorized');
  return applicantService.startApplicant(id);
});

module.exports.putSwapReason = combineResolvers(
  isAuthenticated,
  async (_, { swapReason }, context) => assignPropertyUpdateStatus('swapReason', swapReason, context)
);

module.exports.setLastActive = combineResolvers(
  isAuthenticated,
  async (_, { applicantId }, context) => assignProperty('lastActive', moment.now(), applicantId, context)
);

module.exports.putMember = combineResolvers(
  isAuthenticated,
  async (_, { member }, context) => {
    const data = lodash.omit(member, ['index']);
    const property = member.index === 'ONE' ? 'member1' : 'member2';
    return mergePropertyUpdateStatus(property, data, context);
  }
);

module.exports.deleteMember2 = combineResolvers(
  isAuthenticated,
  async (_, { applicantId }, context) => {
    return assignProperty('member2', null, applicantId, context);
  }
);

module.exports.putApartment = combineResolvers(
  isAuthenticated,
  async (_, { apartment }, context) => {
    const data = lodash.omit(apartment, ['index']);
    const property = apartment.index === 'ONE' ? 'apartment1' : 'apartment2';
    return mergePropertyUpdateStatus(property, data, context);
  }
);

module.exports.putDetails = combineResolvers(
  isAuthenticated,
  async (_, {
    details1,
    details2,
    swapDate,
    applicantId,
  }, context) => {
    details1.applicantId = applicantId;
    details2.applicantId = applicantId;
    const data1 = lodash.omit(details1, ['index']);
    const data2 = lodash.omit(details2, ['index']);
    const resp1 = await mergePropertyUpdateStatus('details1', data1, context);
    const resp2 = await mergePropertyUpdateStatus('details2', data2, context);
    const swapResp = await assignProperty('swapDate', swapDate, applicantId, context);
    const response = {
      applicantId,
      details1: resp1,
      details2: resp2,
      swapDate: swapResp,
    };
    return response;
  }
);

module.exports.approveMember = combineResolvers(isAdmin, (_, { applicantId, index }, context) =>
  applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'member1' : 'member2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'APPROVED',
  }));

module.exports.approveApartment = combineResolvers(isAdmin, (_, { applicantId, index }, context) =>
  applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'apartment1' : 'apartment2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'APPROVED',
  }));

module.exports.approveDetails = combineResolvers(isAdmin, (_, { applicantId, index }, context) =>
  applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'details1' : 'details2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'APPROVED',
  }));


module.exports.approveSwapReason = combineResolvers(isAdmin, (_, { applicantId }, context) =>
  applicantService.setApprovalStatus(applicantId, 'swapReason', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'APPROVED',
  }));

module.exports.rejectMember = combineResolvers(isAdmin, (_, { applicantId, index, message }, context) => {
  checkAndSendRejectionMail(applicantId);

  return applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'member1' : 'member2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'REJECTED',
    message,
  });
});

module.exports.rejectApartment = combineResolvers(isAdmin, (_, { applicantId, index, message }, context) => {
  checkAndSendRejectionMail(applicantId);

  return applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'apartment1' : 'apartment2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'REJECTED',
    message,
  });
});

module.exports.rejectDetails = combineResolvers(isAdmin, (_, { applicantId, index, message }, context) => {
  checkAndSendRejectionMail(applicantId);

  return applicantService.setApprovalStatus(applicantId, index === 'ONE' ? 'details1' : 'details2', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'REJECTED',
    message,
  });
});

module.exports.rejectSwapReason = combineResolvers(isAdmin, (_, { applicantId, message }, context) => {
  checkAndSendRejectionMail(applicantId);

  return applicantService.setApprovalStatus(applicantId, 'swapReason', {
    admin: lodash.get(context, 'user.sub'),
    date: Date.now(),
    type: 'REJECTED',
    message,
  });
});

module.exports.reviewApplicant = combineResolvers(isAdmin, (_, { applicantId }) =>
  applicantService.setProperty(applicantId, 'reviewed', true));

module.exports.setMemberSignLink = combineResolvers(
  isAdmin,
  async (_, { applicantId, index, link }) => {
    const property = index === 'ONE' ? 'member1' : 'member2';
    const applicant = await applicantService.get(applicantId);
    if (!applicant) throw Error('Applicant not found');
    applicant[property].signLink = link;
    await applicantService.putApplicant(applicant);
    // Sending mail
    if (index === 'ONE' && process.env.STAGE === 'prod') {
      console.log(`Sending signlink email to ${applicant.email}`);
      baSendSignLinkEmail(applicant.email, applicant.member1.name, link);
    }

    return applicant[property];
  }
);

module.exports.signApplicant = combineResolvers(isAdmin, (_, { applicantId }) => {
  applicantService.setProperty(applicantId, 'signed', true)
    .then(() => checkAndSendAllSigned(applicantId));
});


module.exports.unSignApplicant = combineResolvers(isAdmin, (_, { applicantId }) =>
  applicantService.setProperty(applicantId, 'signed', false));

module.exports.Applicant = {
  application: applicant => applicationService.get(applicant.application),
  documents: applicant => documentService.getByApplicant(applicant.id),
  bankId: combineResolvers(selfOrAdmin, obj => obj.bankId),
  email: combineResolvers(selfOrAdmin, obj => obj.email),
  members: (obj) => {
    const members = {
      member1: obj.member1,
      member2: obj.member2,
    };
    return members;
  },
  apartments: (obj) => {
    const apartments = {
      apartment1: obj.apartment1,
      apartment2: obj.apartment2,
    };
    return apartments;
  },
  details: (obj) => {
    const details = {
      details1: obj.details1,
      details2: obj.details2,
      swapDate: obj.swapDate,
    };
    return details;
  },

};

module.exports.SwapReason = {
  reason: combineResolvers(selfOrAdmin, obj => obj.reason),
  freeText: combineResolvers(selfOrAdmin, obj => obj.freeText),
};

module.exports.BasicInformation = {
  name: combineResolvers(selfOrAdmin, obj => obj.name),
  ssn: combineResolvers(selfOrAdmin, obj => obj.ssn),
  phone: combineResolvers(selfOrAdmin, obj => obj.phone),
  email: combineResolvers(selfOrAdmin, obj => obj.email),
};

module.exports.ApartmentInformation = {
};

module.exports.Details = {
  member: async (obj) => {
    if (obj.applicantId === null || obj.applicantId === undefined) return null;
    const applicant = await applicantService.get(obj.applicantId);
    const result = obj.memberIndex === 'ONE' ? lodash.get(applicant, 'member1') : lodash.get(applicant, 'member2');
    return result;
  },
  apartment: async (obj) => {
    if (obj.apartmentId === null || obj.apartmentId === undefined) return null;
    const applicant = await applicantService.get(obj.apartmentId);
    const result = obj.apartmentIndex === 'ONE' ? lodash.get(applicant, 'apartment1') : lodash.get(applicant, 'apartment2');
    return result;
  },
};

module.exports.Member = {
  ssn: combineResolvers(selfOrAdmin, obj => obj.ssn),
};

module.exports.Apartment = {
};

