const lodash = require('lodash');
const { combineResolvers } = require('graphql-resolvers');
const applicantService = require('../services/applicantService');
const applicationService = require('../services/applicationService');

const { isAdmin, isAuthenticated } = require('../util/authResolvers');

const assignProperty = async (property, data, applicantId, context) => {
  const applicant = await applicantService.get(applicantId);
  if (!applicant) throw Error('Applicant not found');
  if (lodash.get(context, 'user.email') !== applicant.email) throw Error('Not authorized');
  const application = await applicationService.get(applicant.application);
  application[property] = data;
  await applicationService.put(application);
  return data;
};

module.exports.adminApplications = combineResolvers(isAdmin, (_, { limit, nextToken }) =>
  applicationService.getAll(limit, nextToken));

module.exports.adminApplication = combineResolvers(isAdmin, (_, { id }) =>
  applicationService.get(id));

module.exports.applications = combineResolvers(
  isAuthenticated,
  async (_, args, context) => {
    const applicants = await applicantService.getByEmail(lodash.get(context, 'user.email'));
    await applicantService.setCognitoId(applicants, lodash.get(context, 'user.sub'));
    if (applicants.length === 0) throw Error('No applications for this user');
    const applicationIds = applicants.map(a => a.application);
    if (!applicationIds) throw Error('No applications for this user');
    return applicationService.getMultiple(applicationIds);
  }
);

module.exports.application = combineResolvers(
  isAuthenticated,
  async (_, { id }, context) => {
    const applicants = await applicantService.getByEmail(lodash.get(context, 'user.email'));
    await applicantService.setCognitoId(applicants, lodash.get(context, 'user.sub'));
    const applicant = applicants.find(a => a.id === id);
    if (!applicant) throw Error('Applicant not found');
    if (applicant.cognitoId !== lodash.get(context, 'user.sub')) throw Error('Not Authorized');
    return applicationService.get(applicant.application);
  }
);

module.exports.inactivateApplication = combineResolvers(
  isAuthenticated,
  async (_, { applicantId }, context) => assignProperty('inactivatedBy', applicantId, applicantId, context)
);

module.exports.reactivateApplication = combineResolvers(
  isAuthenticated,
  async (_, { applicantId }, context) => assignProperty('inactivatedBy', null, applicantId, context)
);

module.exports.setApplicationStatus = combineResolvers(isAdmin, (_, { applicationId, status }) =>
  applicationService.setProperty(applicationId, 'status', status));

module.exports.putApplication = combineResolvers(isAuthenticated, (_, { applicants }) =>
  applicationService.putApplication({ applicants: { applicants } }));

module.exports.Application = {
  applicants: a => applicantService.getByApplication(a.id),
};
