const lodash = require('lodash');
const { combineResolvers } = require('graphql-resolvers');
const applicantService = require('../services/applicantService');
const documentService = require('../services/documentService');
const { checkAndSendRejectionMail } = require('../util/mails');

const { isAdmin, isAuthenticated, selfOrAdmin } = require('../util/authResolvers');

module.exports.adminRemoveDocument = combineResolvers(
  isAdmin,
  async (_, { documentId }) => {
    const document = await documentService.get(documentId);
    if (!document) throw Error('Document not found');
    const applicant = await applicantService.get(document.applicant);
    lodash.remove(applicant.documents, id => id === documentId);
    await applicantService.put(applicant);
    return documentService.delete(documentId);
  }
);

module.exports.approveDocument = combineResolvers(isAdmin, (_, { documentId }, context) =>
  documentService.approve(lodash.get(context, 'user.sub'), documentId));

module.exports.rejectDocument = combineResolvers(isAdmin, async (_, { documentId, message }, context) => {
  const document = await documentService.get(documentId);

  checkAndSendRejectionMail(document.applicant);

  return documentService.reject(lodash.get(context, 'user.sub'), documentId, message);
});

module.exports.removeDocument = combineResolvers(
  isAuthenticated,
  async (_, { documentId }, context) => {
    const document = await documentService.get(documentId);
    if (!document || document.cognitoId !== lodash.get(context, 'user.sub')) throw Error('Not authorized');
    await documentService.removeFromS3(document.fileName, document.folder);
    return documentService.delete(documentId);
  }
);

module.exports.uploadDocument = combineResolvers(
  isAuthenticated,
  async (_, { document }, context) => {
    const applicants = await applicantService.getByEmail(lodash.get(context, 'user.email'));
    if (applicants.length === 0) throw Error('Applicant not found');
    console.log(document);
    const applicant = applicants.find(a => a.id === document.applicantId);
    if (!applicant || (lodash.get(context, 'user.email') !== applicant.email)) throw Error('Not authorized');
    const returnedDocument = await documentService.put({
      applicant: applicant.id,
      cognitoId: lodash.get(context, 'user.sub'),
      email: lodash.get(context, 'user.email'),
      folder: document.folder,
      fileName: document.fileName,
      type: document.type,
      uploadedAt: Date.now(),
    });
    applicant.rejectNotified = false; // allow rejection emails again
    if (applicant.documents) applicant.documents.push(returnedDocument.id);
    else applicant.documents = [returnedDocument.id];
    await applicantService.putApplicant(applicant);
    return returnedDocument;
  }
);

module.exports.replaceDocument = combineResolvers(
  isAuthenticated,
  async (_, { documentId, applicantId, fileName }, context) => {
    const applicants = await applicantService.getByEmail(lodash.get(context, 'user.email'));
    if (applicants.length === 0) throw Error('Applicant not found');
    const applicant = applicants.find(a => a.id === applicantId);
    if (!applicant || (lodash.get(context, 'user.email') !== applicant.email)) throw Error('Not authorized');
    const document = await documentService.get(documentId);
    let { approvalStatus } = document;
    if (approvalStatus) approvalStatus.type = 'UPDATED';
    else approvalStatus = { type: 'UPDATED' };
    fileName = fileName || document.fileName;
    const returnedDocument = await documentService.put({
      id: documentId,
      approvalStatus,
      applicant: applicant.id,
      cognitoId: lodash.get(context, 'user.sub'),
      email: lodash.get(context, 'user.email'),
      folder: document.folder,
      fileName,
      type: document.type,
      uploadedAt: Date.now(),
    });
    applicant.rejectNotified = false; // allow rejection emails again
    if (applicant.documents) applicant.documents.push(returnedDocument.id);
    else applicant.documents = [returnedDocument.id];
    await applicantService.putApplicant(applicant);
    return returnedDocument;
  }
);

module.exports.documents = combineResolvers(
  isAuthenticated,
  async (_, { applicantId }) => documentService.getByApplicant(applicantId)
);

module.exports.Document = {
  applicant: document => applicantService.get(document.applicant),
  fileName: combineResolvers(selfOrAdmin, obj => obj.fileName),
  folder: combineResolvers(selfOrAdmin, obj => obj.folder),
};
