const typeDefs = ` 
  type Application {
    id: String!
    status: StatusType!
    applicants: [Applicant!]!
    inactivatedBy: String
  }

  type ApplicationConnection {
    items: [Application]
    nextToken: String
  }

  type Applicant {
    id: String!
    index: Int!
    lastActive: String
    image: String
    profilePicture: String
    started: Boolean
    email: String
    application: Application!
    members: Members
    apartments: Apartments
    details: DetailsPair
    swapReason: SwapReason
    documents: [Document]
    bankId: BankId
    signed: Boolean
    reviewed: Boolean
  }

  type Members {
    member1: Member
    member2: Member
  }

  type Apartments {
    apartment1: Apartment
    apartment2: Apartment
  }

  type DetailsPair {
    details1: Details
    details2: Details
    swapDate: String
  }

  type Details {
    member: Member
    apartment: Apartment
    personCount: Int
    childrenCount: Int
    approvalStatus: ApprovalStatus
    freeTextAddress: String
  }
    
  type Member {
    approvalStatus: ApprovalStatus
    name: String
    ssn: String
    phone: String
    email: String
    signLink: String
    registeredAddress: String
    registeredPostalCode: String
    registeredCity: String
    onCurrentContract: Boolean
  }

  type Apartment {
    approvalStatus: ApprovalStatus
    landlord: String
    address: String
    postalCode: String
    city: String
  }

  type SwapReason {
    reason: [String]
    freeText: String
    approvalStatus: ApprovalStatus
  }

  type Document {
    id: String!
    applicant: Applicant!
    type: DocumentType
    fileName: String
    folder: String
    approvalStatus: ApprovalStatus
    uploadedAt: String
  }

  type BankId {
    ssn: String
    date: String
  }

  type ApprovalStatus {
    admin: Admin
    date: String
    type: ApprovalType
    message: String
  }
  
  type Admin {    
    name: String
    picture: String
    email: String
    phone: String
  }

  enum StatusType {
    ACTIVE
    WAITING
    INACTIVE
    DECLINED
  }
  
  enum DocumentType {
    ID_PROOF
    INCOME_BASE
    RENTAL_AGREEMENT
    RENTAL_BASE
  }

  enum ApprovalType {
    APPROVED
    UPDATED
    REJECTED
  }

  enum DuoIndex {
    ONE
    TWO
  }

  input MemberInput {
    applicantId: String!
    index: DuoIndex!
    name: String
    ssn: String
    phone: String
    email: String
    registeredAddress: String
    registeredPostalCode: String
    registeredCity: String
    onCurrentContract: Boolean
  }

  input ApartmentInput {
    applicantId: String!
    index: DuoIndex!
    landlord: String
    address: String
    postalCode: String
    city: String
  }

  input SwapReasonInput {
    applicantId: String!
    reason: [String]
    freeText: String
  }

  input DetailsInput {
    personCount: Int
    childrenCount: Int
    member: DuoIndex
    moveToApplicant: String
    moveToApartment: DuoIndex
    freeTextAddress: String
    swapDate: String
  }

  input ApplicantInput {
    email: String!
    id: String!
    index: Int!
    application: String!
    documents: [String]
    bankId: BankIdInput
  }

  input ApplicationApplicantInput {
    id: String!
    email: String!
  }

  input BankIdInput {
    ssn: String
    date: String
  }

  input DocumentInput {
    applicantId: String!
    fileName: String!
    folder: String!
    type: DocumentType!
  }

  # the schema allows the following query:
  type Query {
    applications: [Application]
    application(id: String): Application
    applicants: [Applicant]
    adminApplications(limit: Int!, nextToken: String): ApplicationConnection
    adminApplication(id: String!): Application
    documents(applicantId: String!): [Document]
  }
  
  # this schema allows the following mutation:
  type Mutation {
    """ User mutations """
    setLastActive(applicantId: String!): String
    putMember(member: MemberInput): Member 
    deleteMember2(applicantId: String!): String   
    putApartment(apartment: ApartmentInput): Apartment
    putDetails(details1: DetailsInput, details2: DetailsInput, swapDate: String, applicantId: String): DetailsPair
    putSwapReason(swapReason: SwapReasonInput!): SwapReason
    uploadDocument(document: DocumentInput): Document
    replaceDocument(documentId: String!, applicantId: String!, fileName: String): Document
    removeDocument(documentId: String): String
    startApplicant(id: String!): String
    reactivateApplication(applicantId: String!): String
    inactivateApplication(applicantId: String!): String

    """ Admin mutations """
    adminAnonymizeApplicant(email: String!): [Applicant]
    adminRemoveDocument(documentId: String): String
    
    approveDocument(documentId: String!): Document
    approveMember(applicantId: String!, index: DuoIndex!): Member
    approveApartment(applicantId: String!, index: DuoIndex!): Apartment
    approveDetails(applicantId: String!, index: DuoIndex!): Details
    approveSwapReason(applicantId: String!): SwapReason
        
    putApplication(applicants: [ApplicationApplicantInput!]!): Application
    putApplicant(applicant: ApplicantInput): Applicant
    setApplicationStatus(applicationId: String!, status: StatusType): Application
    
    rejectDocument(documentId: String!, message: String): Document
    rejectMember(applicantId: String!, index: DuoIndex!, message: String): Member
    rejectApartment(applicantId: String!, index: DuoIndex!, message: String): Apartment
    rejectDetails(applicantId: String!, index: DuoIndex!, message: String): Details
    rejectSwapReason(applicantId: String!, message: String): SwapReason
    reviewApplicant(applicantId: String!): Applicant
    unReviewApplicant(applicantId: String!): Applicant

    setMemberSignLink(applicantId: String!, index: DuoIndex!, link: String!): Member
    signApplicant(applicantId: String!): Applicant
    unSignApplicant(applicantId: String!): Applicant
  }
`;

module.exports = typeDefs;
