const ServiceBase = require('./serviceBase');
const documentService = require('./documentService');
const uuid = require('uuid/v4');
const lodash = require('lodash');

class ApplicantService extends ServiceBase {
  constructor() {
    super(lodash.get(process, 'env.APPLICANTS_TABLE'));
  }

  async delete(id) {
    await this.dynamoDb.delete({
      TableName: this.tableName,
      Key: { id },
    }).promise();

    return { id };
  }

  async get(id) {
    const response = await this.dynamoDb.get({
      TableName: this.tableName,
      Key: { id },
    }).promise();

    return response.Item;
  }

  async startApplicant(id) {
    const params = {
      TableName: this.tableName,
      Key: { id },
      UpdateExpression: 'set started = :true',
      ExpressionAttributeValues: {
        ':true': true,
      },
      ReturnValues: 'ALL_NEW',
    };
    const response = await this.dynamoDb.update(params).promise();
    return lodash.get(response, 'Attributes.id');
  }


  async getByEmail(email) {
    const response = await this.dynamoDb.query({
      TableName: this.tableName,
      IndexName: 'email-index',
      KeyConditionExpression: 'email = :id',
      ExpressionAttributeValues: {
        ':id': email,
      },
    }).promise();
    return response.Items;
  }

  async getByApplication(application) {
    const response = await this.dynamoDb.query({
      TableName: this.tableName,
      IndexName: 'application-index',
      KeyConditionExpression: 'application = :id',
      ExpressionAttributeValues: {
        ':id': application,
      },
    }).promise();
    return response.Items;
  }

  async setCognitoId(applicants, cognitoId) {
    const promises = [];
    applicants.forEach((applicant) => {
      if (applicant.cognitoId !== cognitoId) {
        applicant.cognitoId = cognitoId;
        if (lodash.get(applicant, 'apartment1')) applicant.apartment1.cognitoId = cognitoId;
        if (lodash.get(applicant, 'apartment2')) applicant.apartment2.cognitoId = cognitoId;
        if (lodash.get(applicant, 'details1')) applicant.details1.cognitoId = cognitoId;
        if (lodash.get(applicant, 'details2')) applicant.details2.cognitoId = cognitoId;
        if (lodash.get(applicant, 'member1')) applicant.member1.cognitoId = cognitoId;
        if (lodash.get(applicant, 'member2')) applicant.member2.cognitoId = cognitoId;
        if (lodash.get(applicant, 'swapReason')) applicant.swapReason.cognitoId = cognitoId;
        promises.push(this.update(applicant));
      }
    });
    const result = await Promise.all(promises);
    return result;
  }

  async update(obj) {
    // this.validateObject(obj, this.schema);

    await this.dynamoDb.put({
      TableName: this.tableName,
      Item: obj,
      ConditionExpression: 'attribute_exists(id)',
    }).promise();

    return obj;
  }

  async putApplicant(obj) {
    if (obj.id === undefined) {
      obj.id = uuid();
    }

    await this.dynamoDb.put({
      TableName: this.tableName,
      Item: obj,
    }).promise();

    return obj;
  }

  async getMultiple(itemIds) {
    const params = {
      RequestItems: {},
    };
    params.RequestItems[this.tableName] = {
      Keys: itemIds.map(id => ({ 'id': id })),
    };
    const response = await this.dynamoDb.batchGet(params).promise();
    return response.Responses[this.tableName];
  }

  async setApprovalStatus(id, property, approvalStatus) {
    const result = await this.dynamoDb.update({
      TableName: this.tableName,
      Key: { id },
      UpdateExpression: 'SET #property.#approvalStatus = :approvalStatus',
      ExpressionAttributeNames: {
        '#property': property,
        '#approvalStatus': 'approvalStatus',
      },
      ExpressionAttributeValues: {
        ':approvalStatus': approvalStatus,
      },
      ReturnValues: 'ALL_NEW',
    }).promise();
    return lodash.get(result, `Attributes.${property}`);
  }

  async anonymizeApplicants(email) {
    const applicants = await this.getByEmail(email);
    for (let i = 0; i < applicants.length; i++) {
      const applicant = applicants[i];
      delete applicant.email;
      delete applicant.image;
      delete applicant.bankId;
      if (applicant.member1) {
        delete applicant.member1.name;
        delete applicant.member1.ssn;
        delete applicant.member1.phone;
        delete applicant.member1.email;
      }
      if (applicant.member2) {
        delete applicant.member2.name;
        delete applicant.member2.ssn;
        delete applicant.member2.phone;
        delete applicant.member2.email;
      }
      const documents = await documentService.getByApplicant(applicant.id);
      for (let j = 0; j < documents.length; j++) {
        const document = documents[j];
        await documentService.removeFromS3(document.fileName, document.folder);
        delete document.fileName;
        delete document.folder;
        await documentService.update(document);
      }
      await this.update(applicant);
    }
    return this.getByEmail(email);
  }
}

module.exports = new ApplicantService();
