const AWS = require('aws-sdk');
const uuid = require('uuid/v4');
const lodash = require('lodash');
// const Joi = require('joi');

module.exports = class ServiceBase {
  constructor(tableName, schema, awsRegion = 'eu-west-1') {
    AWS.config.update({ region: awsRegion });
    this.dynamoDb = new AWS.DynamoDB.DocumentClient({ apiVersion: '2012-10-08' });
    this.tableName = tableName;
    this.schema = schema;
  }

  // validateObject(obj) {
  //   const validationResult = Joi.validate(obj, this.schema);

  //   if (validationResult.error) {
  //     throw Error(validationResult.error);
  //   }
  // }

  async get(id) {
    const response = await this.dynamoDb.get({
      TableName: this.tableName,
      Key: { id },
    }).promise();

    return response.Item;
  }

  async getByCognitoId(cognitoId) {
    const response = await this.dynamoDb.get({
      TableName: this.tableName,
      IndexName: 'cognitoId-index',
      KeyConditionExpression: 'cognitoId = :id',
      ExpressionAttributeValues: {
        ':id': cognitoId,
      },
    }).promise();

    return response.Item;
  }

  async getAll(limit, nextToken) {
    const params = {
      TableName: this.tableName,
      Limit: limit,
    };

    if (nextToken) {
      params.ExclusiveStartKey = {
        id: nextToken,
      };
    }

    const response = await this.dynamoDb.scan(params).promise();
    const getResponse = {
      items: response.Items,
      nextToken: lodash.get(response, 'LastEvaluatedKey.id'),
    };
    return getResponse;
  }

  async getMultiple(itemIds) {
    if (itemIds.length === 0) return null;
    const params = {
      RequestItems: {},
    };
    params.RequestItems[this.tableName] = {
      Keys: itemIds.map(id => ({ 'id': id })),
    };
    const response = await this.dynamoDb.batchGet(params).promise();
    return response.Responses[this.tableName];
  }

  async put(obj) {
    if (obj.id === undefined) {
      obj.id = uuid();
    }

    // if(this.schema) {
    //   this.validateObject(obj);
    // }

    await this.dynamoDb.put({
      TableName: this.tableName,
      Item: obj,
    }).promise();

    return obj;
  }

  async delete(id) {
    await this.dynamoDb.delete({
      TableName: this.tableName,
      Key: { id },
    }).promise();

    return { id };
  }

  async update(obj) {
    // this.validateObject(obj, this.schema);

    await this.dynamoDb.put({
      TableName: this.tableName,
      Item: obj,
      ConditionExpression: 'attribute_exists(id)',
    }).promise();

    return obj;
  }

  async setProperty(id, property, value) {
    const result = await this.dynamoDb.update({
      TableName: this.tableName,
      Key: { id },
      UpdateExpression: 'SET #property = :value',
      ExpressionAttributeNames: {
        '#property': property,
      },
      ExpressionAttributeValues: {
        ':value': value,
      },
      ReturnValues: 'ALL_NEW',
    }).promise();
    return lodash.get(result, 'Attributes');
  }

  async setAsRemoved(id) {
    await this.dynamoDb.update({
      TableName: this.tableName,
      Key: { id },
      UpdateExpression: 'set #removedAt = :now',
      ExpressionAttributeNames: { '#removedAt': 'removedAt' },
      ExpressionAttributeValues: {
        ':now': new Date().toISOString(),
      },
      ReturnValues: 'ALL_NEW',
    }).promise();

    return { id };
  }
};
