const { 
  baSendRejectedEmail, 
  baSendAllSignedEmail, 
  baSendActivationEmail,
  baSendNotifyAdminMail
} = require('@lagenhetsbyte/message-service');
const applicantService = require('../services/applicantService');
const applicationService = require('../services/applicationService');
const documentService = require('../services/documentService');
const get = require('lodash/get');

const sendAllSignedToAll = (applicants) => {
  applicants.forEach(applicant => process.env.STAGE === 'prod' && baSendAllSignedEmail(applicant.email, applicant.member1.name));
};

const checkAllFormsUpdated = applicant => {
  let allUpdated = true;

  // Get all approvalStatuses
  const member1Updated = get(applicant, 'member1.approvalStatus');
  const member2Updated = get(applicant, 'member2.approvalStatus');
  const apartment1Updated = get(applicant, 'apartment1.approvalStatus');
  const apartment2Updated = get(applicant, 'apartment2.approvalStatus');
  const swapReasonUpdated = get(applicant, 'swapReason.approvalStatus');
  const details1Updated = get(applicant, 'details1.approvalStatus');
  const details2Updated = get(applicant, 'details2.approvalStatus');

  //  Check if any approvalStatuses are either null, undefined or REJECTED
  if (!member1Updated || (member1Updated && member1Updated.type === 'REJECTED')) allUpdated = false;
  if (applicant.member2 && (!member2Updated || (member2Updated && member2Updated.type === 'REJECTED'))) allUpdated = false;
  if (!apartment1Updated || (apartment1Updated && apartment1Updated.type === 'REJECTED')) allUpdated = false;
  if (applicant.apartment2 && (!apartment2Updated || (apartment2Updated && apartment2Updated.type === 'REJECTED'))) allUpdated = false;
  if (!swapReasonUpdated || (swapReasonUpdated && swapReasonUpdated.type === 'REJECTED')) allUpdated = false;
  if (!details1Updated || (details1Updated && details1Updated.type === 'REJECTED')) allUpdated = false;
  if (applicant.details2 && (!details2Updated || (details2Updated && details2Updated.type === 'REJECTED'))) allUpdated = false;

  console.log(`Statuscheck for admin mail for applicant ${applicant.id}`, {
    member1Updated, 
    member2Updated, 
    apartment1Updated, 
    apartment2Updated, 
    swapReasonUpdated, 
    details1Updated, 
    details2Updated
  });

  console.log(allUpdated ? 'All forms updated' : 'All forms not updated');
  return allUpdated; 
}

const checkDocumentCategories = async (applicant) => {
  let allCategoriesPresent = true;

  try {
    const documents = await documentService.getByApplicant(applicant.id);

    if (documents.length > 0) {
      const types = documents.map(doc => {
        return doc.type;
      });
      
      if (
        types.indexOf('ID_PROOF') === -1 ||
        types.indexOf('INCOME_BASE') === -1 ||
        types.indexOf('RENTAL_AGREEMENT') === -1 ||
        types.indexOf('RENTAL_BASE') === -1
      ) {
        return false;
      }
    }
    
  } catch (err) {
    console.error(err);
  }

  return true
}

module.exports.checkAndSendRejectionMail = async (applicantId) => {
  const applicant = await applicantService.get(applicantId);
  if (applicant.rejectNotified) return; // Don't send multiple emails for same rejection batch

  baSendRejectedEmail(applicant.email, applicant.member1.name, `https://bytesansokan.lagenhetsbyte.se/?propositionId=${applicantId}`);

  applicant.rejectNotified = true;
  applicantService.putApplicant(applicant);
};

module.exports.sendActivationMail = applicants => {
  applicants.forEach((applicant) => {
    console.log('Checking if should send activation email');
    if (process.env.STAGE === 'prod') {
      console.log(`Sending activation email to ${applicant.email}`);
      baSendActivationEmail(applicant.email, applicant.member1.name, `https://bytesansokan.lagenhetsbyte.se/?propositionid=${applicant.id}`);
    }
  });
}


module.exports.checkAndSendAllSigned = async (applicantId) => {
  const applicant = await applicantService.get(applicantId);
  const application = await applicationService.get(applicant.application);

  const applicants = await applicantService.getByApplication(application.id);
  let allSigned = true;
  applicants.forEach((a) => {
    if (!a.signed) allSigned = false;
  });
  if (allSigned) sendAllSignedToAll(applicants);
};

module.exports.checkAndSendNotifyAdmin = async (applicant) => {
  if (applicant.adminNotified) return;

  const allUpdated = checkAllFormsUpdated(applicant); 
  const allDocumentCategoriesUploaded = await checkDocumentCategories(applicant);

  if(!allUpdated && !allDocumentCategoriesUploaded) {
    console.log('All forms and documents updated!') 
    baSendNotifyAdminMail('andreas@lagenhetsbyte.se', applicant.application, applicant.email, 'adminlink') // send mail to admin

    applicant.adminNotified = true; // set adminNotified to true in application
    newApplicant = await applicantService.putApplicant(applicant) 

    return true;
  } else {
    console.log(`Still stuff left to fill out for ${applicant.id}!`) // dont send mail to admin
    return false;
  }
}